using Microsoft.Extensions.Configuration;
using System.Text.Json;
using System.Text.Json.Serialization;

namespace View_by_Distance.Rename.Models.Binder;

public class RenameConfiguration
{

    public int? EpicYear { get; set; }
    public string[]? IgnoreExtensions { get; set; }
    public string[]? IgnoreRulesKeyWords { get; set; }
    public string[]? ValidImageFormatExtensions { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderConfigurationSourceGenerationContext.Default.RenameConfiguration);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, RenameConfiguration? configuration)
    {
        if (configuration?.EpicYear is null)
        {
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                if (!physicalFileProvider.Root.Contains("UserSecrets"))
                    continue;
                throw new NotSupportedException(physicalFileProvider.Root);
            }
        }
    }

    private static void Verify(RenameConfiguration configuration)
    {
        if (configuration.IgnoreExtensions is null || configuration.IgnoreExtensions.Length == 0) throw new NullReferenceException(nameof(configuration.IgnoreExtensions));
        if (configuration.IgnoreRulesKeyWords is null || configuration.IgnoreRulesKeyWords.Length == 0) throw new NullReferenceException(nameof(configuration.IgnoreRulesKeyWords));
        if (configuration.ValidImageFormatExtensions is null || configuration.ValidImageFormatExtensions.Length == 0) throw new NullReferenceException(nameof(configuration.ValidImageFormatExtensions));
    }

    private static Models.RenameConfiguration Get(RenameConfiguration? configuration, Shared.Models.MetadataConfiguration metadataConfiguration)
    {
        Models.RenameConfiguration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.EpicYear is null) throw new NullReferenceException(nameof(configuration.EpicYear));
        if (configuration.IgnoreExtensions is null) throw new NullReferenceException(nameof(configuration.IgnoreExtensions));
        if (configuration.IgnoreRulesKeyWords is null) throw new NullReferenceException(nameof(configuration.IgnoreRulesKeyWords));
        if (configuration.ValidImageFormatExtensions is null) throw new NullReferenceException(nameof(configuration.ValidImageFormatExtensions));
        Verify(configuration);
        result = new(metadataConfiguration,
                     configuration.EpicYear.Value,
                     configuration.IgnoreExtensions,
                     configuration.IgnoreRulesKeyWords,
                     configuration.ValidImageFormatExtensions);
        return result;
    }

    public static Models.RenameConfiguration Get(IConfigurationRoot configurationRoot, Shared.Models.MetadataConfiguration metadataConfiguration)
    {
        Models.RenameConfiguration result;
        IConfigurationSection configurationSection = configurationRoot.GetSection(nameof(RenameConfiguration));
#pragma warning disable IL3050, IL2026
        RenameConfiguration? configuration = configurationSection.Get<RenameConfiguration>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, configuration);
        result = Get(configuration, metadataConfiguration);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(RenameConfiguration))]
internal partial class BinderConfigurationSourceGenerationContext : JsonSerializerContext
{
}